#import "PDFConverter.h"
#import "filename.h"

#define MENUTAG_FILE			10000
#define MENUTAG_FILE_OPEN		10001
#define MENUTAG_FILE_CONVERT	10002

@implementation PDFConverter

- (IBAction)open:(id)sender
{
	NSOpenPanel *panel = [NSOpenPanel openPanel];
	NSArray *fileTypes = [NSArray arrayWithObjects:@"pdf",nil];
	NSString *dir = lastOpenDir ? lastOpenDir : NSHomeDirectory();
	
	int result = [panel runModalForDirectory:dir file:nil types:fileTypes];
	if (result != NSOKButton)
		return;

	if (lastOpenDir)
		[lastOpenDir release];
	lastOpenDir = [[panel directory] retain];

	NSString *fileName = [panel filename];

	if (pdfDoc) {
		[pdfDoc release];
		pdfDoc = nil;
	}
	pdfDoc = [[PDFDocument alloc] initWithURL: [NSURL fileURLWithPath: fileName]];
	if (!pdfDoc)
		goto error;

	NSData *dataPage = [[pdfDoc pageAtIndex: 0] dataRepresentation];
	NSPDFImageRep *pdfImageRep = [[[NSPDFImageRep alloc] initWithData:dataPage] autorelease];
	if (!pdfImageRep)
		goto release_and_error;

		
	pdfWidth  = [pdfImageRep pixelsWide];
	pdfHeight = [pdfImageRep pixelsHigh];
	if (pdfWidth == 0 || pdfHeight == 0)
		goto release_and_error;
	pdfRatio = (double) pdfWidth / (double) pdfHeight;
	
	[textPageNum setStringValue: [NSString stringWithFormat: @"%d", [pdfDoc pageCount]]];
	[textWidth   setStringValue: [NSString stringWithFormat: @"%d px", pdfWidth]];
	[textHeight  setStringValue: [NSString stringWithFormat: @"%d px", pdfHeight]];
	[textFieldWidth   setStringValue: [NSString stringWithFormat: @"%d", pdfWidth]];
	[textFieldHeight  setStringValue: [NSString stringWithFormat: @"%d", pdfHeight]];

	[textFieldWidth setEnabled:YES];
	[textFieldHeight setEnabled:YES];

	return;

release_and_error:
	[pdfDoc release];
	pdfDoc = nil;

	[textFieldWidth setEnabled:NO];
	[textFieldHeight setEnabled:NO];
	
error:
	NSRunAlertPanel(@"Alert", 
			NSLocalizedString(@"ErrorCantOpenDocument", nil), 
			@"OK", nil, nil);
}

- (IBAction)convert:(id)sender
{
	if (!pdfDoc) {
		NSRunAlertPanel(@"Alert", 
					NSLocalizedString(@"ErrorNoDocument", nil),
					@"OK", nil, nil);
		return;
	}
	// Xbhւ̃p[^
	outWidth  = [textFieldWidth intValue];
	outHeight = [textFieldHeight intValue];
	if (outWidth <= 0 || outHeight <= 0) {
		NSRunAlertPanel(@"Alert", 
					NSLocalizedString(@"ErrorSizeTooSmall", nil),
					@"OK", nil, nil);
		return;
	}
	if (outWidth > 2000 || outHeight > 2000) {
		NSRunAlertPanel(@"Alert", 
					NSLocalizedString(@"ErrorSizeTooLarge", nil),
					@"OK", nil, nil);
		return;
	}
	
	NSSavePanel *panel = [NSSavePanel savePanel];
	int result = [panel runModal];
	if (result != NSOKButton) {
		return;
	}

	[self lockUI];

	threadIsRunning = TRUE;
	needAbort = FALSE;
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(onThreadExit)
		name:NSThreadWillExitNotification object:nil];

	NSDictionary *arg = [NSDictionary dictionaryWithObjectsAndKeys:
							[panel directory], @"savePath",
							[panel filename],  @"filename",
							nil];
	[NSThread detachNewThreadSelector:@selector(_convert_thread:)
										toTarget:self withObject:arg];
}

- (IBAction)onAbort:(id)sender
{
	needAbort = TRUE;
}

- (void) onThreadExit
{
	NSLog(@"thread exit");

	if (!threadIsRunning) {
		[[NSNotificationCenter defaultCenter] removeObserver:self
				name:NSThreadWillExitNotification
				object:nil];

		[self unlockUI];
	}
}

- (void) lockUI
{
	[btnOpen    setEnabled:NO];
	[btnConvert setEnabled:NO];

	NSMenuItem *fileItem = [[NSApp mainMenu] itemWithTag: MENUTAG_FILE];
	[[fileItem submenu] setAutoenablesItems:NO];
	[[[fileItem submenu] itemWithTag: MENUTAG_FILE_OPEN] setEnabled:NO];
	[[[fileItem submenu] itemWithTag: MENUTAG_FILE_CONVERT] setEnabled:NO];
}

- (void) unlockUI
{
	[btnOpen    setEnabled:YES];
	[btnConvert setEnabled:YES];

	NSMenuItem *fileItem = [[NSApp mainMenu] itemWithTag: MENUTAG_FILE];
	[[[fileItem submenu] itemWithTag: MENUTAG_FILE_OPEN] setEnabled:YES];
	[[[fileItem submenu] itemWithTag: MENUTAG_FILE_CONVERT] setEnabled:YES];
}

- (void)controlTextDidChange:(NSNotification *)aNotification
{
	if (!pdfDoc)
		return;
		
	NSTextField *field = [aNotification object];
	// cۂ
	if (field == textFieldWidth) {
		int h = [textFieldWidth doubleValue] / pdfRatio;
		[textFieldHeight setIntValue:h];
	} else if (field == textFieldHeight) {
		int w = pdfRatio * [textFieldHeight intValue];
		[textFieldWidth setIntValue:w];
	}
}

- (BOOL)windowShouldClose:(id)sender
{
    [NSApp terminate:self];
    return YES;
}

- (void)sheetDidEnd:(NSWindow*)sheet 
                returnCode:(int)returnCode 
                contextInfo:(void*)contextInfo
{
    [sheet close];
}

- (void) _convert_thread:(NSDictionary *) arg
{
	NSAutoreleasePool* pool;
    pool = [[NSAutoreleasePool alloc] init];

	[NSApp beginSheet:progressWindow 
			modalForWindow:window
			modalDelegate:self 
			didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:) 
			contextInfo:nil];
	[indicator setDoubleValue:0.0];

	[self _pdf2jpeg:pdfDoc path:[arg objectForKey:@"savePath"]
						file:[arg objectForKey:@"filename"]];

	[NSApp endSheet:progressWindow];

	[pool release];

	threadIsRunning = NO;

	[NSThread exit];
}

- (BOOL) _pdf2jpeg:(PDFDocument *)doc path:(NSString *)savePath file:(NSString *)filename
{
	int i;
	char basename[1024];
	char ext[100];

	filename2name([filename UTF8String], basename, sizeof(basename));
	filename2ext([filename UTF8String], ext, sizeof(ext));

	NSDictionary *propJpeg;
	propJpeg = [NSDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithFloat: 0.5],
				NSImageCompressionFactor,
				nil];

	int pageN = [doc pageCount];
	for (i = 0 ; i < pageN ; i++) {
		NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
		PDFPage *page = [doc pageAtIndex: i];
		NSImage *image = [self _pdf2image: page];
		if (!image) {
			[pool release];
			return NO;
		}

		// Bitmap摜ɕϊ
		NSData *tiffRep = [image TIFFRepresentation];
		NSBitmapImageRep *imageRep = [[[NSBitmapImageRep alloc] initWithData:tiffRep] autorelease];
		if (!imageRep) {
			[pool release];
			return NO;
		}
	
		// Jpego
		NSData       *dataJpeg;
		BOOL          bResult;
		dataJpeg = [imageRep representationUsingType: NSJPEGFileType properties:propJpeg];

		NSString *filename = [NSString stringWithFormat: @"%s/%s-%d.%s",
								[savePath UTF8String], basename, i, ext];
		bResult = [dataJpeg writeToFile : filename atomically: YES];
		if (!bResult)
			NSLog(@"ERR");

		[progressMsg setStringValue:[NSLocalizedString(@"Processing", nil)
				stringByAppendingFormat:@"...(%d/%d)", i+1, pageN]];
		[indicator setDoubleValue:(i + 1)*100 / pageN];
		
		[pool release];

		if (needAbort)
			break;
	}

	[progressMsg setStringValue:@""];

	return YES;
}

- (NSImage *)_pdf2image:(PDFPage *)page
{
	NSData *dataPage = [page dataRepresentation];

	// PDF摜 (NSBitmapImageRepƃTCYɉ摜rȂ)
	NSPDFImageRep *pdfImageRep = [[[NSPDFImageRep alloc] initWithData:dataPage] autorelease];
	if (!pdfImageRep)
		return nil;

	// NSImageɏ
	NSSize size;	// o̓TCY
	if (1) {
		size.width  = outWidth;
		size.height = outHeight;
	} else {
		// {
		size.width  = [pdfImageRep pixelsWide];
		size.height = [pdfImageRep pixelsHigh];
	}

	NSImage *image = [[[NSImage alloc] initWithSize:size] autorelease];
	if (!image)
		return nil;
	[image lockFocus];
	[pdfImageRep drawInRect: NSMakeRect(0, 0, size.width, size.height)];
	[image unlockFocus];

	return image;
}

@end
