#include "filename.h"

#define PATH_DELIMITER	'/'

static char * _basename_pos(const char *filename)
{
	const char *pos;

	pos = strrchr(filename, PATH_DELIMITER);
	if (pos)
		pos++;
	else
		pos = filename;

	return (char *) pos;
}

int filename2path(const char *filename, char *path, size_t n)
{
	const char *end;
	size_t len;

	end = strrchr(filename, PATH_DELIMITER);
	if (!end) {
		strncpy(path, "./", n - 1);
		path[n - 1] = '\0';
		return 0;
	}

	len = end - filename + 1;
	if (n - 1 < len) {
		len = n - 1;
	}

	strncpy(path, filename, len);
	path[len] = '\0';

	return 1;
}

int filename2basename(const char *filename, char *basename, size_t n)
{
	const char *start;

	start = _basename_pos(filename);

	strncpy(basename, start, n - 1);
	basename[n - 1] = '\0';

	return 1;
}

int filename2name(const char *filename, char *name, size_t n)
{
	const char *start;
	const char *name_end;
	size_t len;

	start = _basename_pos(filename);

	name_end = strrchr(start, '.');
	if (!name_end)
		len = strlen(start);

	len = name_end - start;
	if (n - 1 < len)
		len = n - 1;

	strncpy(name, start, len);
	name[len] = '\0';

	return 1;
}

int filename2ext(const char *filename, char *ext, size_t n)
{
	const char *start;
	const char *ext_start;

	start = _basename_pos(filename);

	ext_start = strrchr(start, '.');
	if (!ext_start) {
		ext[0] = '\0';
		return 0;
	}

	strncpy(ext, ext_start + 1, n - 1);
	ext[n - 1] = '\0';

	return 1;
}
